[dpdk-dev] [PATCH 4/4] app/testpmd: add tos and ttl field to vxlan encapsulation

2018-12-29 Thread Viacheslav Ovsiienko
The new testpmd set vxlan-tos-ttl command is added. It
allows to specify tos and tll fields for encapsulation IP
header.

IPv4 VXLAN outer header:

  testpmd> set vxlan-tos-ttl ip-version ipv4 vni 4 udp-src 4
   udp-dst 4 ip-tos 0 ip-ttl 255 ip-src 127.0.0.1
   ip-dst 128.0.0.1 eth-src 11:11:11:11:11:11
   eth-dst 22:22:22:22:22:22

IPv6 VXLAN outer header:
  testpmd> set vxlan-tos-ttl ip-version ipv6 vni 4 udp-src 4
   udp-dst 4 ip-tos 0 ip-ttl 255 ::1 ip-dst ::
   eth-src 11:11:11:11:11:11 eth-dst
   22:22:22:22:22:22

Note: ip-ttl parameter corresponds the nop_limits field for IPv6.

Signed-off-by: Viacheslav Ovsiienko 
---
 app/test-pmd/cmdline.c  | 63 +
 app/test-pmd/cmdline_flow.c | 32 +++
 app/test-pmd/testpmd.c  |  3 ++
 app/test-pmd/testpmd.h  |  3 ++
 doc/guides/testpmd_app_ug/testpmd_funcs.rst | 16 
 5 files changed, 117 insertions(+)

diff --git a/app/test-pmd/cmdline.c b/app/test-pmd/cmdline.c
index 3ddc3e0..9e9e898 100644
--- a/app/test-pmd/cmdline.c
+++ b/app/test-pmd/cmdline.c
@@ -794,6 +794,12 @@ static void cmd_help_long_parsed(void *parsed_result,
" eth-dst (eth-dst)\n"
"   Configure the VXLAN encapsulation for 
flows.\n\n"
 
+   "vxlan-tos-ttl ip-version (ipv4|ipv6) vni (vni) udp-src"
+   " (udp-src) udp-dst (udp-dst) ip-tos (ip-tos) ip-ttl 
(ip-ttl)"
+   " ip-src (ip-src) ip-dst (ip-dst) eth-src (eth-src)"
+   " eth-dst (eth-dst)\n"
+   "   Configure the VXLAN encapsulation for 
flows.\n\n"
+
"nvgre ip-version (ipv4|ipv6) tni (tni) ip-src"
" (ip-src) ip-dst (ip-dst) eth-src (eth-src) eth-dst"
" (eth-dst)\n"
@@ -15034,6 +15040,8 @@ struct cmd_set_vxlan_result {
cmdline_ipaddr_t ip_src;
cmdline_ipaddr_t ip_dst;
uint16_t tci;
+   uint8_t tos;
+   uint8_t ttl;
struct ether_addr eth_src;
struct ether_addr eth_dst;
 };
@@ -15042,6 +15050,9 @@ struct cmd_set_vxlan_result {
TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, set, "set");
 cmdline_parse_token_string_t cmd_set_vxlan_vxlan =
TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, vxlan, "vxlan");
+cmdline_parse_token_string_t cmd_set_vxlan_vxlan_tos_ttl =
+   TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, vxlan,
+"vxlan-tos-ttl");
 cmdline_parse_token_string_t cmd_set_vxlan_vxlan_with_vlan =
TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, vxlan,
 "vxlan-with-vlan");
@@ -15066,6 +15077,16 @@ struct cmd_set_vxlan_result {
 "udp-dst");
 cmdline_parse_token_num_t cmd_set_vxlan_udp_dst_value =
TOKEN_NUM_INITIALIZER(struct cmd_set_vxlan_result, udp_dst, UINT16);
+cmdline_parse_token_string_t cmd_set_vxlan_ip_tos =
+   TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, pos_token,
+"ip-tos");
+cmdline_parse_token_num_t cmd_set_vxlan_ip_tos_value =
+   TOKEN_NUM_INITIALIZER(struct cmd_set_vxlan_result, tos, UINT8);
+cmdline_parse_token_string_t cmd_set_vxlan_ip_ttl =
+   TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, pos_token,
+"ip-ttl");
+cmdline_parse_token_num_t cmd_set_vxlan_ip_ttl_value =
+   TOKEN_NUM_INITIALIZER(struct cmd_set_vxlan_result, ttl, UINT8);
 cmdline_parse_token_string_t cmd_set_vxlan_ip_src =
TOKEN_STRING_INITIALIZER(struct cmd_set_vxlan_result, pos_token,
 "ip-src");
@@ -15104,10 +15125,15 @@ static void cmd_set_vxlan_parsed(void *parsed_result,
.vxlan_id = rte_cpu_to_be_32(res->vni) & RTE_BE32(0x00ff),
};
 
+   vxlan_encap_conf.select_tos = 0;
if (strcmp(res->vxlan, "vxlan") == 0)
vxlan_encap_conf.select_vlan = 0;
else if (strcmp(res->vxlan, "vxlan-with-vlan") == 0)
vxlan_encap_conf.select_vlan = 1;
+   else if (strcmp(res->vxlan, "vxlan-tos-ttl") == 0) {
+   vxlan_encap_conf.select_vlan = 0;
+   vxlan_encap_conf.select_tos = 1;
+   }
if (strcmp(res->ip_version, "ipv4") == 0)
vxlan_encap_conf.select_ipv4 = 1;
else if (strcmp(res->ip_version, "ipv6") == 0)
@@ -15117,6 +15143,8 @@ static void cmd_set_vxlan_parsed(void *parsed_result,
rte_memcpy(vxlan_encap_conf.vni, &id.vni[1], 3);
vxlan_encap_conf.udp_src = rte_cpu_to_be_16(res->udp_src);
vxlan_encap_conf.udp_dst = rte_cpu_to_be_16(res->udp_dst);
+   vxlan_encap_conf.ip_tos = res->tos;
+   vxlan_encap_conf.ip_ttl = res->ttl;
if (vxlan_encap_conf.select_ipv4) {
 

[dpdk-dev] [PATCH 1/4] net/mlx5: add tos and ttl flower match and tunnel keys

2018-12-29 Thread Viacheslav Ovsiienko
This patch is a preparation for adding the type-of-service and
time-to-live IP header fields support on E-Switch. There are
two types of keys added - one for match pattern, other for
tunnel encapsulation header.

This issue is critical for some Open VSwitch configuration
on overlayed (tunneled) networks, where the tos field can be
inherited from outer header to inner header.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/Makefile| 50 
 drivers/net/mlx5/meson.build | 20 
 drivers/net/mlx5/mlx5_flow_tcf.c | 35 
 3 files changed, 105 insertions(+)

diff --git a/drivers/net/mlx5/Makefile b/drivers/net/mlx5/Makefile
index 895cdfe..6246f49 100644
--- a/drivers/net/mlx5/Makefile
+++ b/drivers/net/mlx5/Makefile
@@ -373,6 +373,26 @@ mlx5_autoconf.h.new: $(RTE_SDK)/buildtools/auto-config-h.sh
enum TCA_FLOWER_KEY_TCP_FLAGS_MASK \
$(AUTOCONF_OUTPUT)
$Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_IP_TOS \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_IP_TOS \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_IP_TOS_MASK \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_IP_TOS_MASK \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_IP_TTL \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_IP_TTL \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_IP_TTL_MASK \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_IP_TTL_MASK \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
HAVE_TC_ACT_GOTO_CHAIN \
linux/pkt_cls.h \
define TC_ACT_GOTO_CHAIN \
@@ -448,6 +468,26 @@ mlx5_autoconf.h.new: $(RTE_SDK)/buildtools/auto-config-h.sh
enum TCA_FLOWER_KEY_ENC_UDP_DST_PORT_MASK \
$(AUTOCONF_OUTPUT)
$Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_ENC_IP_TOS \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_ENC_IP_TOS \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_ENC_IP_TOS_MASK \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_ENC_IP_TOS_MASK \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_ENC_IP_TTL \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_ENC_IP_TTL \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_FLOWER_KEY_ENC_IP_TTL_MASK \
+   linux/pkt_cls.h \
+   enum TCA_FLOWER_KEY_ENC_IP_TTL_MASK \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
HAVE_TC_ACT_TUNNEL_KEY \
linux/tc_act/tc_tunnel_key.h \
define TCA_ACT_TUNNEL_KEY \
@@ -458,6 +498,16 @@ mlx5_autoconf.h.new: $(RTE_SDK)/buildtools/auto-config-h.sh
enum TCA_TUNNEL_KEY_ENC_DST_PORT \
$(AUTOCONF_OUTPUT)
$Q sh -- '$<' '$@' \
+   HAVE_TCA_TUNNEL_KEY_ENC_TOS \
+   linux/tc_act/tc_tunnel_key.h \
+   enum TCA_TUNNEL_KEY_ENC_TOS \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
+   HAVE_TCA_TUNNEL_KEY_ENC_TTL \
+   linux/tc_act/tc_tunnel_key.h \
+   enum TCA_TUNNEL_KEY_ENC_TTL \
+   $(AUTOCONF_OUTPUT)
+   $Q sh -- '$<' '$@' \
HAVE_TCA_TUNNEL_KEY_NO_CSUM \
linux/tc_act/tc_tunnel_key.h \
enum TCA_TUNNEL_KEY_NO_CSUM \
diff --git a/drivers/net/mlx5/meson.build b/drivers/net/mlx5/meson.build
index 28938db..d64ae1b 100644
--- a/drivers/net/mlx5/meson.build
+++ b/drivers/net/mlx5/meson.build
@@ -192,6 +192,14 @@ if build
'TCA_FLOWER_KEY_TCP_FLAGS' ],
[ 'HAVE_TCA_FLOWER_KEY_TCP_FLAGS_MASK', 'linux/pkt_cls.h',
'TCA_FLOWER_KEY_TCP_FLAGS_MASK' ],
+   [ 'HAVE_TCA_FLOWER_KEY_IP_TOS', 'linux/pkt_cls.h',
+   'TCA_FLOWER_KEY_IP_TOS' ],
+   [ 'HAVE_TCA_FLOWER_KEY_IP_TOS_MASK', 'linux/pkt_cls.h',
+   'TCA_FLOWER_KEY_IP_TOS_MASK' ],
+   [ 'HAVE_TCA_FLOWER_KEY_IP_TTL', 'linux/pkt_cls.h',
+   'TCA_FLOWER_KEY_IP_TTL' ],
+   [ 'HAVE_TCA_FLOWER_KEY_IP_TTL_MASK', 'linux/pkt_cls.h',
+   'TCA_FLOWER_KEY_IP_TTL_MASK' ],
[ 'HAVE_TC_ACT_GOTO_CHAIN', 'linux/pkt_cls.h',
'TC_ACT_GOTO_CHAIN' ],
[ 'HAVE_TC_ACT_VLAN', 'linux/tc_act/tc_vlan.h',
@@ -222,10 +230,22 @@ if build
'TCA_FLOWER_KEY_ENC_UDP_DST_PORT' ],
[ 'HAVE_TCA_FLOWER_KEY_ENC_UDP_DST_PORT_MASK', 
'linux/pkt_cls.h',
'TCA_FLOWER_KEY_ENC_UDP

[dpdk-dev] [PATCH 2/4] net/mlx5: add tos and ttl fields support on E-Switch

2018-12-29 Thread Viacheslav Ovsiienko
This patch adds the type-of-service and time-to-live IP header
fields support on E-Switch. There match pattern for both fields
with masking is added. Also these fields can be set for VXLAN
tunnel encapsulation header.

This issue is critical for some Open VSwitch configuration
on overlayed (tunneled) networks, where the tos field can be
inherited from outer header to inner header.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 166 +--
 1 file changed, 160 insertions(+), 6 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index 87585ed..ca8ea0b 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -372,6 +372,8 @@ enum flow_tcf_tunact_type {
 #define FLOW_TCF_ENCAP_UDP_SRC (1u << 6)
 #define FLOW_TCF_ENCAP_UDP_DST (1u << 7)
 #define FLOW_TCF_ENCAP_VXLAN_VNI (1u << 8)
+#define FLOW_TCF_ENCAP_IP_TTL (1u << 9)
+#define FLOW_TCF_ENCAP_IP_TOS (1u << 10)
 
 /**
  * Structure for holding netlink context.
@@ -457,6 +459,8 @@ struct flow_tcf_vxlan_decap {
 struct flow_tcf_vxlan_encap {
struct flow_tcf_tunnel_hdr hdr;
uint32_t mask;
+   uint8_t ip_tos;
+   uint8_t ip_ttl_hop;
struct {
struct ether_addr dst;
struct ether_addr src;
@@ -1303,6 +1307,20 @@ struct pedit_parser {
  " must be specified for"
  " vxlan encapsulation");
}
+   if (mask->hdr.type_of_service &&
+   mask->hdr.type_of_service != 0xff)
+   return rte_flow_error_set(error, ENOTSUP,
+ RTE_FLOW_ERROR_TYPE_ITEM_MASK, mask,
+ "no support for partial mask on"
+ " \"ipv4.hdr.type_of_service\" field"
+ " for vxlan encapsulation");
+   if (mask->hdr.time_to_live &&
+   mask->hdr.time_to_live != 0xff)
+   return rte_flow_error_set(error, ENOTSUP,
+ RTE_FLOW_ERROR_TYPE_ITEM_MASK, mask,
+ "no support for partial mask on"
+ " \"ipv4.hdr.time_to_live\" field"
+ " for vxlan encapsulation");
return 0;
 }
 
@@ -1324,6 +1342,7 @@ struct pedit_parser {
 {
const struct rte_flow_item_ipv6 *spec = item->spec;
const struct rte_flow_item_ipv6 *mask = item->mask;
+   uint8_t msk6;
 
if (!spec) {
/*
@@ -1389,6 +1408,20 @@ struct pedit_parser {
  " must be specified for"
  " vxlan encapsulation");
}
+   msk6 = (rte_be_to_cpu_32(mask->hdr.vtc_flow) >>
+   IPV6_HDR_TC_SHIFT) & 0xff;
+   if (msk6 && msk6 != 0xff)
+   return rte_flow_error_set(error, ENOTSUP,
+ RTE_FLOW_ERROR_TYPE_ITEM_MASK, mask,
+ "no support for partial mask on"
+ " \"ipv6.hdr.vtc_flow.tos\" field"
+ " for vxlan encapsulation");
+   if (mask->hdr.hop_limits && mask->hdr.hop_limits != 0xff)
+   return rte_flow_error_set(error, ENOTSUP,
+ RTE_FLOW_ERROR_TYPE_ITEM_MASK, mask,
+ "no support for partial mask on"
+ " \"ipv6.hdr.hop_limits\" field"
+ " for vxlan encapsulation");
return 0;
 }
 
@@ -2476,16 +2509,31 @@ struct pedit_parser {
SZ_NLATTR_TYPE_OF(uint8_t) + /* VLAN prio. */
SZ_NLATTR_TYPE_OF(uint16_t); /* VLAN ID. */
break;
-   case RTE_FLOW_ITEM_TYPE_IPV4:
+   case RTE_FLOW_ITEM_TYPE_IPV4: {
+   const struct rte_flow_item_ipv4 *ipv4 = items->mask;
+
size += SZ_NLATTR_TYPE_OF(uint8_t) + /* IP proto. */
SZ_NLATTR_TYPE_OF(uint32_t) * 4;
/* dst/src IP addr and mask. */
+   if (ipv4 && ipv4->hdr.time_to_live)
+   size += SZ_NLATTR_TYPE_OF(uint8_t) * 2;
+   if (ipv4 && ipv4->hdr.type_of_service)
+   size += SZ_NLATTR_TYPE_OF(uint8_t) * 2;
break;
-   case RTE_FLOW_ITEM_TYPE_IPV6:
+   }
+   case RTE_FLOW_ITEM_TYPE_IPV6: {
+   const struct rte_flow_item_ipv6 *ipv6 = items->mask;
+
size += SZ_NLATTR_TYPE_OF(uint8_t) + /* IP proto. */
   

[dpdk-dev] [PATCH 0/4] net/mlx5: add tos and ttl flower match and tunnel keys

2018-12-29 Thread Viacheslav Ovsiienko
This patchset adds the type-of-service and time-to-live IP header
fields (hop-limits and vtc-flow for IPv6) support on E-Switch.
There are two types of keys added - one for match pattern, other for
tunnel encapsulation header.

Signed-off-by: Viacheslav Ovsiienko 

Viacheslav Ovsiienko (4):
  net/mlx5: add tos and ttl flower match and tunnel keys
  net/mlx5: add tos and ttl fields support on E-Switch
  net/mlx5: add tos and ttl validation on E-Switch
  app/testpmd: add tos and ttl field to vxlan encapsulation

 app/test-pmd/cmdline.c  |  63 
 app/test-pmd/cmdline_flow.c |  32 
 app/test-pmd/testpmd.c  |   3 +
 app/test-pmd/testpmd.h  |   3 +
 doc/guides/testpmd_app_ug/testpmd_funcs.rst |  16 ++
 drivers/net/mlx5/Makefile   |  50 +++
 drivers/net/mlx5/meson.build|  20 +++
 drivers/net/mlx5/mlx5_flow.c|  14 +-
 drivers/net/mlx5/mlx5_flow.h|   2 +
 drivers/net/mlx5/mlx5_flow_dv.c |   4 +-
 drivers/net/mlx5/mlx5_flow_tcf.c| 225 ++--
 drivers/net/mlx5/mlx5_flow_verbs.c  |   4 +-
 12 files changed, 416 insertions(+), 20 deletions(-)

-- 
1.8.3.1



[dpdk-dev] [PATCH 3/4] net/mlx5: add tos and ttl validation on E-Switch

2018-12-29 Thread Viacheslav Ovsiienko
This patch adds the type-of-service and time-to-live IP header
fields validation on E-Switch, both for match pattern and
VXLAN encapsulation action IP header itesm. The E-Switch flows
will use the common mlx5_flow_validate_item_ipv4/6 routines
with added extra parameter, specifying the supported fields
mask.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow.c   | 14 --
 drivers/net/mlx5/mlx5_flow.h   |  2 ++
 drivers/net/mlx5/mlx5_flow_dv.c|  4 ++--
 drivers/net/mlx5/mlx5_flow_tcf.c   | 24 
 drivers/net/mlx5/mlx5_flow_verbs.c |  4 ++--
 5 files changed, 34 insertions(+), 14 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow.c b/drivers/net/mlx5/mlx5_flow.c
index ee129b9..0fd6ed5 100644
--- a/drivers/net/mlx5/mlx5_flow.c
+++ b/drivers/net/mlx5/mlx5_flow.c
@@ -1141,6 +1141,9 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
  *   Item specification.
  * @param[in] item_flags
  *   Bit-fields that holds the items detected until now.
+ * @param[in] acc_mask
+ *   Acceptable mask, if NULL default internal default mask
+ *   will be used to check whether item fields are supported.
  * @param[out] error
  *   Pointer to error structure.
  *
@@ -1150,6 +1153,7 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
 int
 mlx5_flow_validate_item_ipv4(const struct rte_flow_item *item,
 uint64_t item_flags,
+const struct rte_flow_item_ipv4 *acc_mask,
 struct rte_flow_error *error)
 {
const struct rte_flow_item_ipv4 *mask = item->mask;
@@ -1185,7 +1189,8 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
  "partial mask is not supported"
  " for protocol");
ret = mlx5_flow_item_acceptable(item, (const uint8_t *)mask,
-   (const uint8_t *)&nic_mask,
+   acc_mask ? (const uint8_t *)acc_mask
+: (const uint8_t *)&nic_mask,
sizeof(struct rte_flow_item_ipv4),
error);
if (ret < 0)
@@ -1200,6 +1205,9 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
  *   Item specification.
  * @param[in] item_flags
  *   Bit-fields that holds the items detected until now.
+ * @param[in] acc_mask
+ *   Acceptable mask, if NULL default internal default mask
+ *   will be used to check whether item fields are supported.
  * @param[out] error
  *   Pointer to error structure.
  *
@@ -1209,6 +1217,7 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
 int
 mlx5_flow_validate_item_ipv6(const struct rte_flow_item *item,
 uint64_t item_flags,
+const struct rte_flow_item_ipv6 *acc_mask,
 struct rte_flow_error *error)
 {
const struct rte_flow_item_ipv6 *mask = item->mask;
@@ -1243,7 +1252,8 @@ uint32_t mlx5_flow_adjust_priority(struct rte_eth_dev 
*dev, int32_t priority,
if (!mask)
mask = &rte_flow_item_ipv6_mask;
ret = mlx5_flow_item_acceptable(item, (const uint8_t *)mask,
-   (const uint8_t *)&nic_mask,
+   acc_mask ? (const uint8_t *)acc_mask
+: (const uint8_t *)&nic_mask,
sizeof(struct rte_flow_item_ipv6),
error);
if (ret < 0)
diff --git a/drivers/net/mlx5/mlx5_flow.h b/drivers/net/mlx5/mlx5_flow.h
index 4a7c052..8e4eacb 100644
--- a/drivers/net/mlx5/mlx5_flow.h
+++ b/drivers/net/mlx5/mlx5_flow.h
@@ -381,9 +381,11 @@ int mlx5_flow_validate_item_gre(const struct rte_flow_item 
*item,
struct rte_flow_error *error);
 int mlx5_flow_validate_item_ipv4(const struct rte_flow_item *item,
 uint64_t item_flags,
+const struct rte_flow_item_ipv4 *acc_mask,
 struct rte_flow_error *error);
 int mlx5_flow_validate_item_ipv6(const struct rte_flow_item *item,
 uint64_t item_flags,
+const struct rte_flow_item_ipv6 *acc_mask,
 struct rte_flow_error *error);
 int mlx5_flow_validate_item_mpls(struct rte_eth_dev *dev,
 const struct rte_flow_item *item,
diff --git a/drivers/net/mlx5/mlx5_flow_dv.c b/drivers/net/mlx5/mlx5_flow_dv.c
index 1f31874..ab87165 100644
--- a/drivers/net/mlx5/mlx5_flow_dv.c
+++ b/drivers/net/mlx5/mlx5_flow_dv.c
@@ -808,7 +808,7 @@
break;
 

[dpdk-dev] [PATCH] net/mlx5: resolve typos and code style issues

2018-12-29 Thread Viacheslav Ovsiienko
This patch fixes typos and codestyle issues in mlx5_flow_tcf.c file

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 22 +++---
 1 file changed, 11 insertions(+), 11 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index 16fc936..33ebddd 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -475,7 +475,7 @@ struct flow_tcf_vxlan_encap {
uint8_t src[IPV6_ADDR_LEN];
} ipv6;
};
-struct {
+   struct {
rte_be16_t src;
rte_be16_t dst;
} udp;
@@ -1338,7 +1338,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  **/
 static int
 flow_tcf_validate_vxlan_encap_ipv6(const struct rte_flow_item *item,
@@ -1439,7 +1439,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  **/
 static int
 flow_tcf_validate_vxlan_encap_udp(const struct rte_flow_item *item,
@@ -1507,7 +1507,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  **/
 static int
 flow_tcf_validate_vxlan_encap_vni(const struct rte_flow_item *item,
@@ -1555,7 +1555,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  **/
 static int
 flow_tcf_validate_vxlan_encap(const struct rte_flow_action *action,
@@ -1737,7 +1737,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  */
 static int
 flow_tcf_validate(struct rte_eth_dev *dev,
@@ -2808,7 +2808,7 @@ struct pedit_parser {
  *
  * @return
  *   Pointer to mlx5_flow object on success,
- *   otherwise NULL and rte_ernno is set.
+ *   otherwise NULL and rte_errno is set.
  */
 static struct mlx5_flow *
 flow_tcf_prepare(const struct rte_flow_attr *attr,
@@ -3214,7 +3214,7 @@ struct pedit_parser {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  */
 static int
 flow_tcf_translate(struct rte_eth_dev *dev, struct mlx5_flow *dev_flow,
@@ -4571,8 +4571,8 @@ struct tcf_nlcb_context {
 
 /**
  * Cleanup the outer interface. Removes all found vxlan devices
- * attached to specified index, flushes the meigh and local IP
- * datavase.
+ * attached to specified index, flushes the neigh and local IP
+ * database.
  *
  * @param[in] tcf
  *   Context object initialized by mlx5_flow_tcf_context_create().
@@ -5580,7 +5580,7 @@ struct tcf_nlcb_query {
  *   Pointer to the error structure.
  *
  * @return
- *   0 on success, a negative errno value otherwise and rte_ernno is set.
+ *   0 on success, a negative errno value otherwise and rte_errno is set.
  */
 static int
 flow_tcf_apply(struct rte_eth_dev *dev, struct rte_flow *flow,
-- 
1.8.3.1



[dpdk-dev] [PATCH 0/5] net/mlx5: simplify VXLAN devices management for E-Switch

2018-12-29 Thread Viacheslav Ovsiienko
This patchset simplifies the virtual VXLAN tunnel devices management.
Previous design used the VXLAN devices attached to outer interface for
encapsulation rules. The new design uses the unattached devices, it allows
use the single VXLAN device both for encapsulation and decapsulation rules
and removes UDP port sharing issues. 

Also patchset introduces the minor changes in VXLAN device management
allowing to be compiled and operate on some old kernels (for example RH7.2
original kernel 3.10.327), which do not support VXLAN device metadata.

Signed-off-by: Viacheslav Ovsiienko 

Viacheslav Ovsiienko (5):
  net/mlx5: optimize neigh and local encap rules search
  net/mlx5: introduce encapsulation rules container
  net/mlx5: switch encap rules to use container
  net/mlx5: switch to detached VXLAN network devices
  net/mlx5: add RH7.2 VXLAN device metadata workaround

 drivers/net/mlx5/mlx5_flow_tcf.c | 270 +++
 1 file changed, 159 insertions(+), 111 deletions(-)

-- 
1.8.3.1



[dpdk-dev] [PATCH 2/5] net/mlx5: introduce encapsulation rules container

2018-12-29 Thread Viacheslav Ovsiienko
Currently the VXLAN encapsulation neigh/local rules
are stored in the list contained in the VTEP device
structure. Encapsulation VTEP device is attached to
outer interface and stored rules are related to this
underlying interface. We are going to use unattached
VXLAN devices for encapsulation (kernel does not use
attached interface to find egress one), so we should
introduce the structure to keep interface related
neigh/local rules instead of VTEP structure. This
patch introduces internal tcf_irule structure, and
its create/delete methods.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 108 ++-
 1 file changed, 107 insertions(+), 1 deletion(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index b4734a0..a6dca08 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -431,6 +431,15 @@ struct tcf_local_rule {
};
 };
 
+/** Outer interface VXLAN encapsulation rules container. */
+struct tcf_irule {
+   LIST_ENTRY(tcf_irule) next;
+   LIST_HEAD(, tcf_neigh_rule) neigh;
+   LIST_HEAD(, tcf_local_rule) local;
+   uint32_t refcnt;
+   unsigned int ifouter; /**< Own interface index. */
+};
+
 /** VXLAN virtual netdev. */
 struct tcf_vtep {
LIST_ENTRY(tcf_vtep) next;
@@ -458,6 +467,7 @@ struct flow_tcf_vxlan_decap {
 
 struct flow_tcf_vxlan_encap {
struct flow_tcf_tunnel_hdr hdr;
+   struct tcf_irule *iface;
uint32_t mask;
uint8_t ip_tos;
uint8_t ip_ttl_hop;
@@ -4971,11 +4981,90 @@ struct tcf_nlcb_context {
return 0;
 }
 
+/* VXLAN encap rule database for outer interfaces. */
+static  LIST_HEAD(, tcf_irule) iface_list_vxlan = LIST_HEAD_INITIALIZER();
+
 /* VTEP device list is shared between PMD port instances. */
 static LIST_HEAD(, tcf_vtep) vtep_list_vxlan = LIST_HEAD_INITIALIZER();
 static pthread_mutex_t vtep_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /**
+ * Acquire the VXLAN encap rules container for specified interface.
+ * First looks for the container in the existing ones list, creates
+ * and initializes the new container if existing not found.
+ *
+ * @param[in] tcf
+ *   Context object initialized by mlx5_flow_tcf_context_create().
+ * @param[in] ifouter
+ *   Network interface index to create VXLAN encap rules on.
+ * @param[out] error
+ *   Perform verbose error reporting if not NULL.
+ * @return
+ *   Rule container pointer on success,
+ *   NULL otherwise and rte_errno is set.
+ */
+static struct tcf_irule*
+flow_tcf_encap_irule_acquire(struct mlx5_flow_tcf_context *tcf,
+unsigned int ifouter,
+struct rte_flow_error *error)
+{
+   struct tcf_irule *iface;
+
+   /* Look whether the container for encap rules is created. */
+   assert(ifouter);
+   LIST_FOREACH(iface, &iface_list_vxlan, next) {
+   if (iface->ifouter == ifouter)
+   break;
+   }
+   if (iface) {
+   /* Container already exists, just increment the reference. */
+   iface->refcnt++;
+   return iface;
+   }
+   /* Not found, we should create the new container. */
+   iface = rte_zmalloc(__func__, sizeof(*iface),
+   alignof(struct tcf_irule));
+   if (!iface) {
+   rte_flow_error_set(error, ENOMEM,
+  RTE_FLOW_ERROR_TYPE_UNSPECIFIED, NULL,
+  "unable to allocate memory for container");
+   return NULL;
+   }
+   *iface = (struct tcf_irule){
+   .local = LIST_HEAD_INITIALIZER(),
+   .neigh = LIST_HEAD_INITIALIZER(),
+   .ifouter = ifouter,
+   .refcnt = 1,
+   };
+   /* Interface cleanup for new container created. */
+   flow_tcf_encap_iface_cleanup(tcf, ifouter);
+   flow_tcf_encap_local_cleanup(tcf, ifouter);
+   flow_tcf_encap_neigh_cleanup(tcf, ifouter);
+   LIST_INSERT_HEAD(&iface_list_vxlan, iface, next);
+   return iface;
+}
+
+/**
+ * Releases VXLAN encap rules container by pointer. Decrements the
+ * reference cointer and deletes the container if counter is zero.
+ *
+ * @param[in] irule
+ *   VXLAN rule container pointer to release.
+ */
+static void
+flow_tcf_encap_irule_release(struct tcf_irule *iface)
+{
+   assert(iface->refcnt);
+   if (--iface->refcnt == 0) {
+   /* Reference counter is zero, delete the container. */
+   assert(LIST_EMPTY(&iface->local));
+   assert(LIST_EMPTY(&iface->neigh));
+   LIST_REMOVE(iface, next);
+   rte_free(iface);
+   }
+}
+
+/**
  * Deletes VTEP network device.
  *
  * @param[in] tcf
@@ -5247,6 +5336,7 @@ struct tcf_nlcb_context {
 {
static uint16_t encap_port = MLX5_VXLAN_PORT_MIN - 1;
struct tcf_vtep *vtep;
+  

[dpdk-dev] [PATCH 1/5] net/mlx5: optimize neigh and local encap rules search

2018-12-29 Thread Viacheslav Ovsiienko
This patch removes unnecessary local varialbles and optimizes
local and neigh encapsulation rules search.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 14 --
 1 file changed, 4 insertions(+), 10 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index 33ebddd..b4734a0 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -4781,8 +4781,7 @@ struct tcf_nlcb_context {
 struct rte_flow_error *error)
 {
const struct flow_tcf_vxlan_encap *encap = dev_flow->tcf.vxlan_encap;
-   struct tcf_local_rule *rule;
-   bool found = false;
+   struct tcf_local_rule *rule = NULL;
int ret;
 
assert(encap);
@@ -4793,7 +4792,6 @@ struct tcf_nlcb_context {
if (rule->mask & FLOW_TCF_ENCAP_IPV4_SRC &&
encap->ipv4.src == rule->ipv4.src &&
encap->ipv4.dst == rule->ipv4.dst) {
-   found = true;
break;
}
}
@@ -4806,12 +4804,11 @@ struct tcf_nlcb_context {
sizeof(encap->ipv6.src)) &&
!memcmp(&encap->ipv6.dst, &rule->ipv6.dst,
sizeof(encap->ipv6.dst))) {
-   found = true;
break;
}
}
}
-   if (found) {
+   if (rule) {
if (enable) {
rule->refcnt++;
return 0;
@@ -4890,8 +4887,7 @@ struct tcf_nlcb_context {
 struct rte_flow_error *error)
 {
const struct flow_tcf_vxlan_encap *encap = dev_flow->tcf.vxlan_encap;
-   struct tcf_neigh_rule *rule;
-   bool found = false;
+   struct tcf_neigh_rule *rule = NULL;
int ret;
 
assert(encap);
@@ -4901,7 +4897,6 @@ struct tcf_nlcb_context {
LIST_FOREACH(rule, &vtep->neigh, next) {
if (rule->mask & FLOW_TCF_ENCAP_IPV4_DST &&
encap->ipv4.dst == rule->ipv4.dst) {
-   found = true;
break;
}
}
@@ -4912,12 +4907,11 @@ struct tcf_nlcb_context {
if (rule->mask & FLOW_TCF_ENCAP_IPV6_DST &&
!memcmp(&encap->ipv6.dst, &rule->ipv6.dst,
sizeof(encap->ipv6.dst))) {
-   found = true;
break;
}
}
}
-   if (found) {
+   if (rule) {
if (memcmp(&encap->eth.dst, &rule->eth,
   sizeof(encap->eth.dst))) {
DRV_LOG(WARNING, "Destination MAC differs"
-- 
1.8.3.1



[dpdk-dev] [PATCH 5/5] net/mlx5: add RH7.2 VXLAN device metadata workaround

2018-12-29 Thread Viacheslav Ovsiienko
RH7.2 with kernel 3.10.0-327 does not support VXLAN
devices metadata and IFLA_VXLAN_COLLECT_METADATA
key is neither defined nor supported. We must specify
VNI parameter, which will be actually ignored by kernel,
applied rules will be processed by mlx5 kernel driver
and the actual VNI from rules will be used.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 33 -
 1 file changed, 16 insertions(+), 17 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index 7f9a76c..72bad85 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -351,9 +351,8 @@ struct tc_tunnel_key {
 #define TCA_ACT_MAX_PRIO 32
 #endif
 
-/** UDP port range of VXLAN devices created by driver. */
-#define MLX5_VXLAN_PORT_MIN 3
-#define MLX5_VXLAN_PORT_MAX 6
+/** Parameters of VXLAN devices created by driver. */
+#define MLX5_VXLAN_DEFAULT_VNI 1
 #define MLX5_VXLAN_DEVICE_PFX "vmlx_"
 
 /** Tunnel action type, used for @p type in header structure. */
@@ -5115,7 +5114,6 @@ struct tcf_nlcb_context {
  * Pointer to created device structure on success,
  * NULL otherwise and rte_errno is set.
  */
-#ifdef HAVE_IFLA_VXLAN_COLLECT_METADATA
 static struct tcf_vtep*
 flow_tcf_vtep_create(struct mlx5_flow_tcf_context *tcf,
 uint16_t port, struct rte_flow_error *error)
@@ -5165,10 +5163,24 @@ struct tcf_nlcb_context {
mnl_attr_put_strz(nlh, IFLA_INFO_KIND, "vxlan");
na_vxlan = mnl_attr_nest_start(nlh, IFLA_INFO_DATA);
assert(na_vxlan);
+#ifdef HAVE_IFLA_VXLAN_COLLECT_METADATA
+   /*
+* RH 7.2 does not support metadata for tunnel device.
+* It does not matter because we are going to use the
+* hardware offload by mlx5 driver.
+*/
mnl_attr_put_u8(nlh, IFLA_VXLAN_COLLECT_METADATA, 1);
+#endif
mnl_attr_put_u8(nlh, IFLA_VXLAN_UDP_ZERO_CSUM6_RX, 1);
mnl_attr_put_u8(nlh, IFLA_VXLAN_LEARNING, 0);
mnl_attr_put_u16(nlh, IFLA_VXLAN_PORT, vxlan_port);
+#ifndef HAVE_IFLA_VXLAN_COLLECT_METADATA
+   /*
+*  We must specify VNI explicitly if metadata not supported.
+*  Note, VNI is transferred with native endianness format.
+*/
+   mnl_attr_put_u16(nlh, IFLA_VXLAN_ID, MLX5_VXLAN_DEFAULT_VNI);
+#endif
mnl_attr_nest_end(nlh, na_vxlan);
mnl_attr_nest_end(nlh, na_info);
assert(sizeof(buf) >= nlh->nlmsg_len);
@@ -5237,19 +5249,6 @@ struct tcf_nlcb_context {
rte_free(vtep);
return NULL;
 }
-#else
-static struct tcf_vtep*
-flow_tcf_vtep_create(struct mlx5_flow_tcf_context *tcf __rte_unused,
-uint16_t port __rte_unused,
-struct rte_flow_error *error)
-{
-   rte_flow_error_set(error, ENOTSUP,
-  RTE_FLOW_ERROR_TYPE_UNSPECIFIED, NULL,
-  "netlink: failed to create VTEP, "
-  "vxlan metadata are not supported by kernel");
-   return NULL;
-}
-#endif /* HAVE_IFLA_VXLAN_COLLECT_METADATA */
 
 /**
  * Acquire target interface index for VXLAN tunneling decapsulation.
-- 
1.8.3.1



[dpdk-dev] [PATCH 4/5] net/mlx5: switch to detached VXLAN network devices

2018-12-29 Thread Viacheslav Ovsiienko
Current design uses the VXLAN virtual devices attached
to outer network interface for decapsulation. Kernel
allows to use non-attached devices, so now we can create
not attached device and use it both for encapsulation
and decapsulation. Devices management becomes simpler,
less VXLAN devices are created and used.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 73 ++--
 1 file changed, 11 insertions(+), 62 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index b99e322..7f9a76c 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -443,11 +443,8 @@ struct tcf_irule {
 /** VXLAN virtual netdev. */
 struct tcf_vtep {
LIST_ENTRY(tcf_vtep) next;
-   LIST_HEAD(, tcf_neigh_rule) neigh;
-   LIST_HEAD(, tcf_local_rule) local;
uint32_t refcnt;
unsigned int ifindex; /**< Own interface index. */
-   unsigned int ifouter; /**< Index of device attached to. */
uint16_t port;
uint8_t created;
 };
@@ -5109,11 +5106,6 @@ struct tcf_nlcb_context {
  *
  * @param[in] tcf
  *   Context object initialized by mlx5_flow_tcf_context_create().
- * @param[in] ifouter
- *   Outer interface to attach new-created VXLAN device
- *   If zero the VXLAN device will not be attached to any device.
- *   These VTEPs are used for decapsulation and can be precreated
- *   and shared between processes.
  * @param[in] port
  *   UDP port of created VTEP device.
  * @param[out] error
@@ -5126,7 +5118,6 @@ struct tcf_nlcb_context {
 #ifdef HAVE_IFLA_VXLAN_COLLECT_METADATA
 static struct tcf_vtep*
 flow_tcf_vtep_create(struct mlx5_flow_tcf_context *tcf,
-unsigned int ifouter,
 uint16_t port, struct rte_flow_error *error)
 {
struct tcf_vtep *vtep;
@@ -5156,8 +5147,6 @@ struct tcf_nlcb_context {
}
*vtep = (struct tcf_vtep){
.port = port,
-   .local = LIST_HEAD_INITIALIZER(),
-   .neigh = LIST_HEAD_INITIALIZER(),
};
memset(buf, 0, sizeof(buf));
nlh = mnl_nlmsg_put_header(buf);
@@ -5175,8 +5164,6 @@ struct tcf_nlcb_context {
assert(na_info);
mnl_attr_put_strz(nlh, IFLA_INFO_KIND, "vxlan");
na_vxlan = mnl_attr_nest_start(nlh, IFLA_INFO_DATA);
-   if (ifouter)
-   mnl_attr_put_u32(nlh, IFLA_VXLAN_LINK, ifouter);
assert(na_vxlan);
mnl_attr_put_u8(nlh, IFLA_VXLAN_COLLECT_METADATA, 1);
mnl_attr_put_u8(nlh, IFLA_VXLAN_UDP_ZERO_CSUM6_RX, 1);
@@ -5190,7 +5177,7 @@ struct tcf_nlcb_context {
DRV_LOG(WARNING,
"netlink: VTEP %s create failure (%d)",
name, rte_errno);
-   if (rte_errno != EEXIST || ifouter)
+   if (rte_errno != EEXIST)
/*
 * Some unhandled error occurred or device is
 * for encapsulation and cannot be shared.
@@ -5216,7 +5203,6 @@ struct tcf_nlcb_context {
goto error;
}
vtep->ifindex = ret;
-   vtep->ifouter = ifouter;
memset(buf, 0, sizeof(buf));
nlh = mnl_nlmsg_put_header(buf);
nlh->nlmsg_type = RTM_NEWLINK;
@@ -5254,7 +5240,6 @@ struct tcf_nlcb_context {
 #else
 static struct tcf_vtep*
 flow_tcf_vtep_create(struct mlx5_flow_tcf_context *tcf __rte_unused,
-unsigned int ifouter __rte_unused,
 uint16_t port __rte_unused,
 struct rte_flow_error *error)
 {
@@ -5293,13 +5278,6 @@ struct tcf_nlcb_context {
if (vtep->port == port)
break;
}
-   if (vtep && vtep->ifouter) {
-   rte_flow_error_set(error, -errno,
-  RTE_FLOW_ERROR_TYPE_UNSPECIFIED, NULL,
-  "Failed to create decap VTEP with specified"
-  " UDP port, atatched device exists");
-   return NULL;
-   }
if (vtep) {
/* Device exists, just increment the reference counter. */
vtep->refcnt++;
@@ -5307,7 +5285,7 @@ struct tcf_nlcb_context {
return vtep;
}
/* No decapsulation device exists, try to create the new one. */
-   vtep = flow_tcf_vtep_create(tcf, 0, port, error);
+   vtep = flow_tcf_vtep_create(tcf, port, error);
if (vtep)
LIST_INSERT_HEAD(&vtep_list_vxlan, vtep, next);
return vtep;
@@ -5319,7 +5297,7 @@ struct tcf_nlcb_context {
  * @param[in] tcf
  *   Context object initialized by mlx5_flow_tcf_context_create().
  * @param[in] ifouter
- *   Network interface index to attach VXLAN encap device to.
+ *   Network interface index to create VXLAN encap rules on.
  * @param[in] dev_flow
  *   Flow tcf object with tunnel structure poin

[dpdk-dev] [PATCH 3/5] net/mlx5: switch encap rules to use container

2018-12-29 Thread Viacheslav Ovsiienko
The VXLAN encapsulation neigh/local rules will use
the new introduced structure, which keeps the
rules lists, related to specified outer interface,
instead of attached VTEP structure. It allows us to
unbind VTEP structure from keeping the rules for
interface.

Signed-off-by: Viacheslav Ovsiienko 
---
 drivers/net/mlx5/mlx5_flow_tcf.c | 42 
 1 file changed, 21 insertions(+), 21 deletions(-)

diff --git a/drivers/net/mlx5/mlx5_flow_tcf.c b/drivers/net/mlx5/mlx5_flow_tcf.c
index a6dca08..b99e322 100644
--- a/drivers/net/mlx5/mlx5_flow_tcf.c
+++ b/drivers/net/mlx5/mlx5_flow_tcf.c
@@ -4771,8 +4771,8 @@ struct tcf_nlcb_context {
  *
  * @param[in] tcf
  *   Libmnl socket context object.
  * @param[in] vtep
  *   VTEP object, contains rule database and ifouter index.
  * @param[in] dev_flow
  *   Flow object, contains the tunnel parameters (for encap only).
  * @param[in] enable
@@ -4785,7 +4785,7 @@ struct tcf_nlcb_context {
  */
 static int
 flow_tcf_encap_local(struct mlx5_flow_tcf_context *tcf,
-struct tcf_vtep *vtep,
+struct tcf_irule *iface,
 struct mlx5_flow *dev_flow,
 bool enable,
 struct rte_flow_error *error)
@@ -4798,7 +4798,7 @@ struct tcf_nlcb_context {
assert(encap->hdr.type == FLOW_TCF_TUNACT_VXLAN_ENCAP);
if (encap->mask & FLOW_TCF_ENCAP_IPV4_SRC) {
assert(encap->mask & FLOW_TCF_ENCAP_IPV4_DST);
-   LIST_FOREACH(rule, &vtep->local, next) {
+   LIST_FOREACH(rule, &iface->local, next) {
if (rule->mask & FLOW_TCF_ENCAP_IPV4_SRC &&
encap->ipv4.src == rule->ipv4.src &&
encap->ipv4.dst == rule->ipv4.dst) {
@@ -4808,7 +4808,7 @@ struct tcf_nlcb_context {
} else {
assert(encap->mask & FLOW_TCF_ENCAP_IPV6_SRC);
assert(encap->mask & FLOW_TCF_ENCAP_IPV6_DST);
-   LIST_FOREACH(rule, &vtep->local, next) {
+   LIST_FOREACH(rule, &iface->local, next) {
if (rule->mask & FLOW_TCF_ENCAP_IPV6_SRC &&
!memcmp(&encap->ipv6.src, &rule->ipv6.src,
sizeof(encap->ipv6.src)) &&
@@ -4826,7 +4826,7 @@ struct tcf_nlcb_context {
if (!rule->refcnt || !--rule->refcnt) {
LIST_REMOVE(rule, next);
return flow_tcf_rule_local(tcf, encap,
-   vtep->ifouter, false, error);
+   iface->ifouter, false, error);
}
return 0;
}
@@ -4859,13 +4859,13 @@ struct tcf_nlcb_context {
memcpy(&rule->ipv6.src, &encap->ipv6.src, IPV6_ADDR_LEN);
memcpy(&rule->ipv6.dst, &encap->ipv6.dst, IPV6_ADDR_LEN);
}
-   ret = flow_tcf_rule_local(tcf, encap, vtep->ifouter, true, error);
+   ret = flow_tcf_rule_local(tcf, encap, iface->ifouter, true, error);
if (ret) {
rte_free(rule);
return ret;
}
rule->refcnt++;
-   LIST_INSERT_HEAD(&vtep->local, rule, next);
+   LIST_INSERT_HEAD(&iface->local, rule, next);
return 0;
 }
 
@@ -4877,8 +4877,8 @@ struct tcf_nlcb_context {
  *
  * @param[in] tcf
  *   Libmnl socket context object.
  * @param[in] vtep
  *   VTEP object, contains rule database and ifouter index.
  * @param[in] dev_flow
  *   Flow object, contains the tunnel parameters (for encap only).
  * @param[in] enable
@@ -4891,7 +4891,7 @@ struct tcf_nlcb_context {
  */
 static int
 flow_tcf_encap_neigh(struct mlx5_flow_tcf_context *tcf,
-struct tcf_vtep *vtep,
+struct tcf_irule *iface,
 struct mlx5_flow *dev_flow,
 bool enable,
 struct rte_flow_error *error)
@@ -4904,7 +4904,7 @@ struct tcf_nlcb_context {
assert(encap->hdr.type == FLOW_TCF_TUNACT_VXLAN_ENCAP);
if (encap->mask & FLOW_TCF_ENCAP_IPV4_DST) {
assert(encap->mask & FLOW_TCF_ENCAP_IPV4_SRC);
-   LIST_FOREACH(rule, &vtep->neigh, next) {
+   LIST_FOREACH(rule, &iface->neigh, next) {
if (rule->mask & FLOW_TCF_ENCAP_IPV4_DST &&
encap->ipv4.dst == rule->ipv4.dst) {
break;
@@ -4913,7 +4913,7 @@ struct tcf_nlcb_context {
} else {
assert(encap->mask & FLOW_TCF_ENCAP_IPV6_SRC);
assert(encap->mask & FLOW_TCF_ENCAP_IPV6_DST);
-   LIST_FOREACH(rule, &vtep->neigh, next) {
+   LIST_FOREACH(rule, &iface->neigh, next) {
@@ -4771,8 +4771,8 @@ struct tcf_nlcb_context {
  * @param[in] tcf
  *   Libmnl socket context object.
- * @param[in] vtep
- *   VTEP object, contains rule database and ifouter index.
+ * @param[in] iface
+ *   Object, contains rule database and ifouter index.
  * @param[in] dev_flow
  *   Flow object, contains the tunnel parameters (for encap only).
@@ -4877,8 +4877,8 @@ struct tcf_nlcb_context {
  * @param[in] tcf
  *   Libmnl socket context object.
- * @param[in] vtep
- *   VTEP object, contains rule database and ifouter index.
+ * @param[in] iface
+ *   Object, contains rule database and ifouter index.
  * @param[in] dev_flow
  *   Flow object, contains the tunnel parameters (for encap only).
if (rule->ma

Re: [dpdk-dev] [PATCH v2 1/3] net/mlx5: fix shared counter allocation logic

2018-12-29 Thread Slava Ovsiienko
Moti, don't you forget to update flow_verbs_counter_release() ?
Only shared counters should be removed from the list.

WBR,
Slava

> -Original Message-
> From: dev  On Behalf Of Mordechay Haimovsky
> Sent: Friday, December 28, 2018 0:20
> To: dev@dpdk.org
> Cc: Mordechay Haimovsky ; sta...@dpdk.org
> Subject: [dpdk-dev] [PATCH v2 1/3] net/mlx5: fix shared counter allocation
> logic
> 
> This commit fixes the logic for searching and allocating a shared counter in
> mlx5_flow_verbs.
> Now only the shared counters in the counters list are checked for a match
> and not all the counters as before.
> 
> Fixes: 84c406e74524 ("net/mlx5: add flow translate function")
> Cc: sta...@dpdk.org
> 
> Signed-off-by: Moti Haimovsky 
> ---
> v2:
> * Modified commit header
> ---
>  drivers/net/mlx5/mlx5_flow_verbs.c | 14 +++---
>  1 file changed, 7 insertions(+), 7 deletions(-)
> 
> diff --git a/drivers/net/mlx5/mlx5_flow_verbs.c
> b/drivers/net/mlx5/mlx5_flow_verbs.c
> index 81ec59d..409e1cd 100644
> --- a/drivers/net/mlx5/mlx5_flow_verbs.c
> +++ b/drivers/net/mlx5/mlx5_flow_verbs.c
> @@ -121,13 +121,13 @@
>   struct mlx5_flow_counter *cnt;
>   int ret;
> 
> - LIST_FOREACH(cnt, &priv->flow_counters, next) {
> - if (!cnt->shared || cnt->shared != shared)
> - continue;
> - if (cnt->id != id)
> - continue;
> - cnt->ref_cnt++;
> - return cnt;
> + if (shared) {
> + LIST_FOREACH(cnt, &priv->flow_counters, next) {
> + if (cnt->shared && cnt->id == id) {
> + cnt->ref_cnt++;
> + return cnt;
> + }
> + }
>   }
>   cnt = rte_calloc(__func__, 1, sizeof(*cnt), 0);
>   if (!cnt) {
> --
> 1.8.3.1